
package com.werken.xpath.impl;

import com.werken.xpath.ContextSupport;

import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.Attribute;
import org.dom4j.Namespace;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collections;

public class NameTestStep extends UnAbbrStep
{

    private String _namespacePrefix = null;
    private String _localName       = null;
  
    public NameTestStep(String axis,
                        String namespacePrefix,
                        String localName)
    {
        super(axis);
        _namespacePrefix = namespacePrefix;
        _localName       = localName;
    }

    public List applyToSelf(Object node,
                            ContextSupport support)
    {
        List results  = new ArrayList();

        if ( node instanceof Element )
        {

            if ( ( "*".equals( _localName ) || ((Element)node).getName().equals(_localName) )
                 &&
                 ((Element)node).getNamespaceURI().equals( support.translateNamespacePrefix( _namespacePrefix ) ) )
            {
                results.add( node );
            }
        }
        else if ( node instanceof Attribute )
        {
            if ( ((Attribute)node).getName().equals(_localName)
                 &&
                 ((Attribute)node).getNamespaceURI().equals( support.translateNamespacePrefix( _namespacePrefix ) ) )
            {
                results.add( node );
            }
        }
    
        return results;
    }
  
    public List applyToAttribute(Object node,
                                 ContextSupport support)
    
    {
        List      results = new ArrayList();
        Attribute attr    = null;
    
        if ( node instanceof Element )
        {
            if ( "*".equals( _localName ) )
            {
                results.addAll( ((Element)node).getAttributes() );
            }
            else
            {
                attr = null;
        
                if ( _namespacePrefix == null )
                {
                    attr = ((Element)node).getAttribute( _localName );
                }
                else
                {
                    Namespace ns = support.getNamespaceByPrefix( _namespacePrefix );

                    attr = ((Element)node).getAttribute( _localName,
                                                         ns );
                }
        
                if ( attr != null )
                {
                    results.add( attr );
                }
            }
        }
    
        return results;
    }
  
    public List applyToChild(Object node,
                             ContextSupport support)
    {
        List results = new ArrayList();
        String nsURI = null;
        Namespace ns = null;

        if ( _namespacePrefix != null )
        {
            ns = support.getNamespaceByPrefix( _namespacePrefix );
        }
        else
        {
            ns = Namespace.NO_NAMESPACE;
        }

        if ( node instanceof Document )
        {
            Element child = ((Document)node).getRootElement();
      
            if ( child.getName().equals( _localName ) )
            {
                if ( ns == Namespace.NO_NAMESPACE )
                {
                    results.add( child );
                }
                else if ( ns.equals( child.getNamespace() ) )
                {
                    results.add( child );
                }
            }
        }
        else if ( node instanceof Element )
        {
            if ( "*".equals( _localName ) )
            {
                List children = ((Element)node).getElements();
        
                if ( ns == Namespace.NO_NAMESPACE )
                {
                    results.addAll( children );
                }
                else
                {
                    Iterator childIter = children.iterator();
                    Element  nodeChild = null;
          
                    while ( childIter.hasNext() )
                    {
                        nodeChild = (Element) childIter.next();
            
                        if ( ns.equals( nodeChild.getNamespace() ) )
                        {
                            results.add( nodeChild );
                        }
                    }
                }
            }
            else
            {
                results.addAll( ((Element)node).getElements( _localName,
                                                             ns ) );
            }
        }

        return results;
    }

    public String toString()
    {
        return "[NameTestStep [" + _namespacePrefix + ":" + _localName + "]]";
    }
}
